@@ -419,7 +419,7 @@ func TestImportStarMangleNoBundleUnused(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.js"},
 		options: config.Options{
-			MangleSyntax:  true,
+			MinifySyntax:  true,
 			AbsOutputFile: "/out.js",
 		},
 	})
@@ -436,7 +436,7 @@ func TestImportStarMangleNoBundleCapture(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.js"},
 		options: config.Options{
-			MangleSyntax:  true,
+			MinifySyntax:  true,
 			AbsOutputFile: "/out.js",
 		},
 	})
@@ -453,7 +453,7 @@ func TestImportStarMangleNoBundleNoCapture(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.js"},
 		options: config.Options{
-			MangleSyntax:  true,
+			MinifySyntax:  true,
 			AbsOutputFile: "/out.js",
 		},
 	})
@@ -1065,7 +1065,8 @@ func TestNamespaceImportMissingES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedCompileLog: `entry.js: WARNING: Import "foo" will always be undefined because there is no matching export in "foo.js"
+		debugLogs: true,
+		expectedCompileLog: `entry.js: DEBUG: Import "foo" will always be undefined because there is no matching export in "foo.js"
 `,
 	})
 }
@@ -1127,7 +1128,8 @@ func TestNamespaceImportUnusedMissingES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedCompileLog: `entry.js: WARNING: Import "foo" will always be undefined because there is no matching export in "foo.js"
+		debugLogs: true,
+		expectedCompileLog: `entry.js: DEBUG: Import "foo" will always be undefined because there is no matching export in "foo.js"
 `,
 	})
 }
@@ -1283,7 +1285,8 @@ func TestNamespaceImportReExportStarMissingES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedCompileLog: `entry.js: WARNING: Import "foo" will always be undefined because there is no matching export in "foo.js"
+		debugLogs: true,
+		expectedCompileLog: `entry.js: DEBUG: Import "foo" will always be undefined because there is no matching export in "foo.js"
 `,
 	})
 }
@@ -1307,7 +1310,8 @@ func TestNamespaceImportReExportStarUnusedMissingES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedCompileLog: `entry.js: WARNING: Import "foo" will always be undefined because there is no matching export in "foo.js"
+		debugLogs: true,
+		expectedCompileLog: `entry.js: DEBUG: Import "foo" will always be undefined because there is no matching export in "foo.js"
 `,
 	})
 }
@@ -1370,10 +1374,10 @@ func TestReExportStarExternalIIFE(t *testing.T) {
 			OutputFormat:  config.FormatIIFE,
 			AbsOutputFile: "/out.js",
 			GlobalName:    []string{"mod"},
-			ExternalModules: config.ExternalModules{
-				NodeModules: map[string]bool{
+			ExternalSettings: config.ExternalSettings{
+				PreResolve: config.ExternalMatchers{Exact: map[string]bool{
 					"foo": true,
-				},
+				}},
 			},
 		},
 	})
@@ -1391,10 +1395,10 @@ func TestReExportStarExternalES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputFile: "/out.js",
-			ExternalModules: config.ExternalModules{
-				NodeModules: map[string]bool{
+			ExternalSettings: config.ExternalSettings{
+				PreResolve: config.ExternalMatchers{Exact: map[string]bool{
 					"foo": true,
-				},
+				}},
 			},
 		},
 	})
@@ -1412,10 +1416,10 @@ func TestReExportStarExternalCommonJS(t *testing.T) {
 			Mode:          config.ModeBundle,
 			OutputFormat:  config.FormatCommonJS,
 			AbsOutputFile: "/out.js",
-			ExternalModules: config.ExternalModules{
-				NodeModules: map[string]bool{
+			ExternalSettings: config.ExternalSettings{
+				PreResolve: config.ExternalMatchers{Exact: map[string]bool{
 					"foo": true,
-				},
+				}},
 			},
 		},
 	})
@@ -1483,10 +1487,10 @@ func TestReExportStarAsExternalIIFE(t *testing.T) {
 			OutputFormat:  config.FormatIIFE,
 			AbsOutputFile: "/out.js",
 			GlobalName:    []string{"mod"},
-			ExternalModules: config.ExternalModules{
-				NodeModules: map[string]bool{
+			ExternalSettings: config.ExternalSettings{
+				PreResolve: config.ExternalMatchers{Exact: map[string]bool{
 					"foo": true,
-				},
+				}},
 			},
 		},
 	})
@@ -1504,10 +1508,10 @@ func TestReExportStarAsExternalES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputFile: "/out.js",
-			ExternalModules: config.ExternalModules{
-				NodeModules: map[string]bool{
+			ExternalSettings: config.ExternalSettings{
+				PreResolve: config.ExternalMatchers{Exact: map[string]bool{
 					"foo": true,
-				},
+				}},
 			},
 		},
 	})
@@ -1525,10 +1529,10 @@ func TestReExportStarAsExternalCommonJS(t *testing.T) {
 			Mode:          config.ModeBundle,
 			OutputFormat:  config.FormatCommonJS,
 			AbsOutputFile: "/out.js",
-			ExternalModules: config.ExternalModules{
-				NodeModules: map[string]bool{
+			ExternalSettings: config.ExternalSettings{
+				PreResolve: config.ExternalMatchers{Exact: map[string]bool{
 					"foo": true,
-				},
+				}},
 			},
 		},
 	})
@@ -1655,14 +1659,15 @@ func TestImportDefaultNamespaceComboIssue446(t *testing.T) {
 		options: config.Options{
 			Mode:         config.ModeBundle,
 			AbsOutputDir: "/out",
-			ExternalModules: config.ExternalModules{
-				NodeModules: map[string]bool{
+			ExternalSettings: config.ExternalSettings{
+				PreResolve: config.ExternalMatchers{Exact: map[string]bool{
 					"external": true,
-				},
+				}},
 			},
 		},
-		expectedCompileLog: `internal-def.js: WARNING: Import "def" will always be undefined because there is no matching export in "internal.js"
-internal-ns-def.js: WARNING: Import "def" will always be undefined because there is no matching export in "internal.js"
+		debugLogs: true,
+		expectedCompileLog: `internal-def.js: DEBUG: Import "def" will always be undefined because there is no matching export in "internal.js"
+internal-ns-def.js: DEBUG: Import "def" will always be undefined because there is no matching export in "internal.js"
 `,
 	})
 }
@@ -1688,13 +1693,14 @@ func TestImportDefaultNamespaceComboNoDefault(t *testing.T) {
 			Mode:         config.ModeBundle,
 			AbsOutputDir: "/out",
 		},
+		debugLogs: true,
 		expectedCompileLog: `entry-default-ns-prop.js: ERROR: No matching export in "foo.js" for import "default"
-entry-default-ns-prop.js: WARNING: Import "default" will always be undefined because there is no matching export in "foo.js"
+entry-default-ns-prop.js: DEBUG: Import "default" will always be undefined because there is no matching export in "foo.js"
 entry-default-ns.js: ERROR: No matching export in "foo.js" for import "default"
 entry-default-prop.js: ERROR: No matching export in "foo.js" for import "default"
-entry-default-prop.js: WARNING: Import "default" will always be undefined because there is no matching export in "foo.js"
+entry-default-prop.js: DEBUG: Import "default" will always be undefined because there is no matching export in "foo.js"
 entry-default.js: ERROR: No matching export in "foo.js" for import "default"
-entry-prop.js: WARNING: Import "default" will always be undefined because there is no matching export in "foo.js"
+entry-prop.js: DEBUG: Import "default" will always be undefined because there is no matching export in "foo.js"
 `,
 	})
 }
@@ -1740,7 +1746,8 @@ func TestImportNamespaceUndefinedPropertyEmptyFile(t *testing.T) {
 			Mode:         config.ModeBundle,
 			AbsOutputDir: "/out",
 		},
-		expectedCompileLog: `entry-default.js: WARNING: Import "default" will always be undefined because there is no matching export in "empty.mjs"
+		debugLogs: true,
+		expectedCompileLog: `entry-default.js: DEBUG: Import "default" will always be undefined because there is no matching export in "empty.mjs"
 entry-nope.js: WARNING: Import "nope" will always be undefined because the file "empty.js" has no exports
 entry-nope.js: WARNING: Import "nope" will always be undefined because the file "empty.mjs" has no exports
 entry-nope.js: WARNING: Import "nope" will always be undefined because the file "empty.cjs" has no exports
@@ -1790,7 +1797,8 @@ func TestImportNamespaceUndefinedPropertySideEffectFreeFile(t *testing.T) {
 			Mode:         config.ModeBundle,
 			AbsOutputDir: "/out",
 		},
-		expectedCompileLog: `entry-default.js: WARNING: Import "default" will always be undefined because there is no matching export in "foo/no-side-effects.mjs"
+		debugLogs: true,
+		expectedCompileLog: `entry-default.js: DEBUG: Import "default" will always be undefined because there is no matching export in "foo/no-side-effects.mjs"
 entry-nope.js: WARNING: Import "nope" will always be undefined because the file "foo/no-side-effects.js" has no exports
 entry-nope.js: WARNING: Import "nope" will always be undefined because the file "foo/no-side-effects.mjs" has no exports
 entry-nope.js: WARNING: Import "nope" will always be undefined because the file "foo/no-side-effects.cjs" has no exports
