@@ -805,6 +805,49 @@ func TestPackageJsonBrowserIndexNoExt(t *testing.T) {
 	})
 }
 
+// See https://github.com/evanw/esbuild/issues/2002
+func TestPackageJsonBrowserIssue2002A(t *testing.T) {
+	packagejson_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.js": `require('pkg/sub')`,
+			"/Users/user/project/src/node_modules/pkg/package.json": `{
+				"browser": {
+					"./sub": "./sub/foo.js"
+				}
+			}`,
+			"/Users/user/project/src/node_modules/pkg/sub/foo.js":   `require('sub')`,
+			"/Users/user/project/src/node_modules/sub/package.json": `{ "main": "./bar" }`,
+			"/Users/user/project/src/node_modules/sub/bar.js":       `works()`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+		},
+	})
+}
+
+func TestPackageJsonBrowserIssue2002B(t *testing.T) {
+	packagejson_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.js": `require('pkg/sub')`,
+			"/Users/user/project/src/node_modules/pkg/package.json": `{
+				"browser": {
+					"./sub": "./sub/foo.js",
+					"./sub/sub": "./sub/bar.js"
+				}
+			}`,
+			"/Users/user/project/src/node_modules/pkg/sub/foo.js": `require('sub')`,
+			"/Users/user/project/src/node_modules/pkg/sub/bar.js": `works()`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+		},
+	})
+}
+
 func TestPackageJsonDualPackageHazardImportOnly(t *testing.T) {
 	packagejson_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -1490,6 +1533,118 @@ func TestPackageJsonExportsDefaultOverImportAndRequire(t *testing.T) {
 	})
 }
 
+func TestPackageJsonExportsEntryPointImportOverRequire(t *testing.T) {
+	packagejson_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/node_modules/pkg/package.json": `
+				{
+					"exports": {
+						"import": "./import.js",
+						"require": "./require.js"
+					},
+					"module": "./module.js",
+					"main": "./main.js"
+				}
+			`,
+			"/node_modules/pkg/import.js": `
+				console.log('SUCCESS')
+			`,
+			"/node_modules/pkg/require.js": `
+				console.log('FAILURE')
+			`,
+			"/node_modules/pkg/module.js": `
+				console.log('FAILURE')
+			`,
+			"/node_modules/pkg/main.js": `
+				console.log('FAILURE')
+			`,
+		},
+		entryPaths: []string{"pkg"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestPackageJsonExportsEntryPointRequireOnly(t *testing.T) {
+	packagejson_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/node_modules/pkg/package.json": `
+				{
+					"exports": {
+						"require": "./require.js"
+					},
+					"module": "./module.js",
+					"main": "./main.js"
+				}
+			`,
+			"/node_modules/pkg/require.js": `
+				console.log('FAILURE')
+			`,
+			"/node_modules/pkg/module.js": `
+				console.log('FAILURE')
+			`,
+			"/node_modules/pkg/main.js": `
+				console.log('FAILURE')
+			`,
+		},
+		entryPaths: []string{"pkg"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+		expectedScanLog: `ERROR: Could not resolve "pkg"
+node_modules/pkg/package.json: NOTE: The path "." is not currently exported by package "pkg":
+node_modules/pkg/package.json: NOTE: None of the conditions provided ("require") match any of the currently active conditions ("browser", "default", "import"):
+`,
+	})
+}
+
+func TestPackageJsonExportsEntryPointModuleOverMain(t *testing.T) {
+	packagejson_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/node_modules/pkg/package.json": `
+				{
+					"module": "./module.js",
+					"main": "./main.js"
+				}
+			`,
+			"/node_modules/pkg/module.js": `
+				console.log('SUCCESS')
+			`,
+			"/node_modules/pkg/main.js": `
+				console.log('FAILURE')
+			`,
+		},
+		entryPaths: []string{"pkg"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestPackageJsonExportsEntryPointMainOnly(t *testing.T) {
+	packagejson_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/node_modules/pkg/package.json": `
+				{
+					"main": "./main.js"
+				}
+			`,
+			"/node_modules/pkg/main.js": `
+				console.log('SUCCESS')
+			`,
+		},
+		entryPaths: []string{"pkg"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
 func TestPackageJsonExportsBrowser(t *testing.T) {
 	packagejson_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -2193,3 +2348,26 @@ NOTE: You can mark the path "foo" as external to exclude it from the bundle, whi
 `,
 	})
 }
+
+func TestPackageJsonTypeShouldBeTypes(t *testing.T) {
+	packagejson_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/index.js": ``,
+			"/Users/user/project/package.json": `
+				{
+					"main": "./src/index.js",
+					"type": "./src/index.d.ts"
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/index.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+			MainFields:    []string{},
+		},
+		expectedScanLog: `Users/user/project/package.json: WARNING: "./src/index.d.ts" is not a valid value for the "type" field
+Users/user/project/package.json: NOTE: TypeScript type declarations use the "types" field, not the "type" field:
+`,
+	})
+}
