@@ -400,8 +400,8 @@ func TestTSMinifyEnum(t *testing.T) {
 		},
 		entryPaths: []string{"/a.ts", "/b.ts"},
 		options: config.Options{
-			MangleSyntax:      true,
-			RemoveWhitespace:  true,
+			MinifySyntax:      true,
+			MinifyWhitespace:  true,
 			MinifyIdentifiers: true,
 			AbsOutputDir:      "/",
 		},
@@ -420,8 +420,8 @@ func TestTSMinifyNestedEnum(t *testing.T) {
 		},
 		entryPaths: []string{"/a.ts", "/b.ts"},
 		options: config.Options{
-			MangleSyntax:      true,
-			RemoveWhitespace:  true,
+			MinifySyntax:      true,
+			MinifyWhitespace:  true,
 			MinifyIdentifiers: true,
 			AbsOutputDir:      "/",
 		},
@@ -440,8 +440,8 @@ func TestTSMinifyNestedEnumNoLogicalAssignment(t *testing.T) {
 		},
 		entryPaths: []string{"/a.ts", "/b.ts"},
 		options: config.Options{
-			MangleSyntax:          true,
-			RemoveWhitespace:      true,
+			MinifySyntax:          true,
+			MinifyWhitespace:      true,
 			MinifyIdentifiers:     true,
 			AbsOutputDir:          "/",
 			UnsupportedJSFeatures: compat.LogicalAssignment,
@@ -461,8 +461,8 @@ func TestTSMinifyNestedEnumNoArrow(t *testing.T) {
 		},
 		entryPaths: []string{"/a.ts", "/b.ts"},
 		options: config.Options{
-			MangleSyntax:          true,
-			RemoveWhitespace:      true,
+			MinifySyntax:          true,
+			MinifyWhitespace:      true,
 			MinifyIdentifiers:     true,
 			AbsOutputDir:          "/",
 			UnsupportedJSFeatures: compat.Arrow,
@@ -490,8 +490,8 @@ func TestTSMinifyNamespace(t *testing.T) {
 		},
 		entryPaths: []string{"/a.ts", "/b.ts"},
 		options: config.Options{
-			MangleSyntax:      true,
-			RemoveWhitespace:  true,
+			MinifySyntax:      true,
+			MinifyWhitespace:  true,
 			MinifyIdentifiers: true,
 			AbsOutputDir:      "/",
 		},
@@ -518,8 +518,8 @@ func TestTSMinifyNamespaceNoLogicalAssignment(t *testing.T) {
 		},
 		entryPaths: []string{"/a.ts", "/b.ts"},
 		options: config.Options{
-			MangleSyntax:          true,
-			RemoveWhitespace:      true,
+			MinifySyntax:          true,
+			MinifyWhitespace:      true,
 			MinifyIdentifiers:     true,
 			AbsOutputDir:          "/",
 			UnsupportedJSFeatures: compat.LogicalAssignment,
@@ -547,8 +547,8 @@ func TestTSMinifyNamespaceNoArrow(t *testing.T) {
 		},
 		entryPaths: []string{"/a.ts", "/b.ts"},
 		options: config.Options{
-			MangleSyntax:          true,
-			RemoveWhitespace:      true,
+			MinifySyntax:          true,
+			MinifyWhitespace:      true,
 			MinifyIdentifiers:     true,
 			AbsOutputDir:          "/",
 			UnsupportedJSFeatures: compat.Arrow,
@@ -573,7 +573,7 @@ func TestTSMinifyDerivedClass(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.ts"},
 		options: config.Options{
-			MangleSyntax:          true,
+			MinifySyntax:          true,
 			UnsupportedJSFeatures: es(2015),
 			AbsOutputFile:         "/out.js",
 		},
@@ -666,8 +666,8 @@ func TestTSMinifiedBundleES6(t *testing.T) {
 		entryPaths: []string{"/entry.ts"},
 		options: config.Options{
 			Mode:              config.ModeBundle,
-			MangleSyntax:      true,
-			RemoveWhitespace:  true,
+			MinifySyntax:      true,
+			MinifyWhitespace:  true,
 			MinifyIdentifiers: true,
 			AbsOutputFile:     "/out.js",
 		},
@@ -693,8 +693,8 @@ func TestTSMinifiedBundleCommonJS(t *testing.T) {
 		entryPaths: []string{"/entry.ts"},
 		options: config.Options{
 			Mode:              config.ModeBundle,
-			MangleSyntax:      true,
-			RemoveWhitespace:  true,
+			MinifySyntax:      true,
+			MinifyWhitespace:  true,
 			MinifyIdentifiers: true,
 			AbsOutputFile:     "/out.js",
 		},
