@@ -30,9 +30,10 @@ type parser struct {
 }
 
 type Options struct {
+	OriginalTargetEnv      string
 	UnsupportedCSSFeatures compat.CSSFeature
-	MangleSyntax           bool
-	RemoveWhitespace       bool
+	MinifySyntax           bool
+	MinifyWhitespace       bool
 }
 
 func Parse(log logger.Log, source logger.Source, options Options) css_ast.AST {
@@ -116,6 +117,9 @@ func (p *parser) expect(kind css_lexer.T) bool {
 		return true
 	}
 	t := p.current()
+	if (t.Flags & css_lexer.DidWarnAboutSingleLineComment) != 0 {
+		return false
+	}
 
 	var text string
 	var suggestion string
@@ -151,7 +155,7 @@ func (p *parser) expect(kind css_lexer.T) bool {
 }
 
 func (p *parser) unexpected() {
-	if t := p.current(); t.Range.Loc.Start > p.prevError.Start {
+	if t := p.current(); t.Range.Loc.Start > p.prevError.Start && (t.Flags&css_lexer.DidWarnAboutSingleLineComment) == 0 {
 		var text string
 		switch t.Kind {
 		case css_lexer.TEndOfFile, css_lexer.TWhitespace:
@@ -173,10 +177,13 @@ type ruleContext struct {
 }
 
 func (p *parser) parseListOfRules(context ruleContext) []css_ast.Rule {
-	didWarnAboutCharset := false
-	didWarnAboutImport := false
+	atRuleContext := atRuleContext{}
+	if context.isTopLevel {
+		atRuleContext.charsetValidity = atRuleValid
+		atRuleContext.importValidity = atRuleValid
+	}
 	rules := []css_ast.Rule{}
-	locs := []logger.Loc{}
+	didFindAtImport := false
 
 loop:
 	for {
@@ -189,9 +196,6 @@ loop:
 			}
 			if comment.TokenIndexAfter == uint32(p.index) {
 				rules = append(rules, css_ast.Rule{Loc: comment.Loc, Data: &css_ast.RComment{Text: comment.Text}})
-				if context.isTopLevel {
-					locs = append(locs, comment.Loc)
-				}
 			}
 			p.legalCommentIndex++
 		}
@@ -210,47 +214,47 @@ loop:
 			continue
 
 		case css_lexer.TAtKeyword:
-			first := p.current().Range
-			rule := p.parseAtRule(atRuleContext{})
+			rule := p.parseAtRule(atRuleContext)
 
-			// Validate structure
+			// Disallow "@charset" and "@import" after other rules
 			if context.isTopLevel {
-				switch rule.Data.(type) {
+				switch r := rule.Data.(type) {
 				case *css_ast.RAtCharset:
-					if !didWarnAboutCharset {
-						for i, before := range rules {
-							if _, ok := before.Data.(*css_ast.RComment); !ok {
-								p.log.AddWithNotes(logger.Warning, &p.tracker, first, "\"@charset\" must be the first rule in the file",
-									[]logger.MsgData{p.tracker.MsgData(logger.Range{Loc: locs[i]},
-										"This rule cannot come before a \"@charset\" rule")})
-								didWarnAboutCharset = true
-								break
-							}
-						}
-					}
+					// This doesn't invalidate anything because it always comes first
 
 				case *css_ast.RAtImport:
-					if !didWarnAboutImport {
-					importLoop:
-						for i, before := range rules {
-							switch before.Data.(type) {
-							case *css_ast.RComment, *css_ast.RAtCharset, *css_ast.RAtImport:
-							default:
-								p.log.AddWithNotes(logger.Warning, &p.tracker, first, "All \"@import\" rules must come first",
-									[]logger.MsgData{p.tracker.MsgData(logger.Range{Loc: locs[i]},
-										"This rule cannot come before an \"@import\" rule")})
-								didWarnAboutImport = true
-								break importLoop
-							}
-						}
+					didFindAtImport = true
+					if atRuleContext.charsetValidity == atRuleValid {
+						atRuleContext.afterLoc = rule.Loc
+						atRuleContext.charsetValidity = atRuleInvalidAfter
+					}
+
+				case *css_ast.RAtLayer:
+					if atRuleContext.charsetValidity == atRuleValid {
+						atRuleContext.afterLoc = rule.Loc
+						atRuleContext.charsetValidity = atRuleInvalidAfter
+					}
+
+					// From the specification: "Note: No @layer rules are allowed between
+					// @import and @namespace rules. Any @layer rule that comes after an
+					// @import or @namespace rule will cause any subsequent @import or
+					// @namespace rules to be ignored."
+					if atRuleContext.importValidity == atRuleValid && (r.Rules != nil || didFindAtImport) {
+						atRuleContext.afterLoc = rule.Loc
+						atRuleContext.charsetValidity = atRuleInvalidAfter
+						atRuleContext.importValidity = atRuleInvalidAfter
+					}
+
+				default:
+					if atRuleContext.importValidity == atRuleValid {
+						atRuleContext.afterLoc = rule.Loc
+						atRuleContext.charsetValidity = atRuleInvalidAfter
+						atRuleContext.importValidity = atRuleInvalidAfter
 					}
 				}
 			}
 
 			rules = append(rules, rule)
-			if context.isTopLevel {
-				locs = append(locs, first.Loc)
-			}
 			continue
 
 		case css_lexer.TCDO, css_lexer.TCDC:
@@ -260,23 +264,27 @@ loop:
 			}
 		}
 
-		if context.isTopLevel {
-			locs = append(locs, p.current().Range.Loc)
+		if atRuleContext.importValidity == atRuleValid {
+			atRuleContext.afterLoc = p.current().Range.Loc
+			atRuleContext.charsetValidity = atRuleInvalidAfter
+			atRuleContext.importValidity = atRuleInvalidAfter
 		}
+
 		if context.parseSelectors {
-			rules = append(rules, p.parseSelectorRule())
+			rules = append(rules, p.parseSelectorRuleFrom(p.index, parseSelectorOpts{}))
 		} else {
 			rules = append(rules, p.parseQualifiedRuleFrom(p.index, false /* isAlreadyInvalid */))
 		}
 	}
 
-	if p.options.MangleSyntax {
+	if p.options.MinifySyntax {
 		rules = mangleRules(rules)
 	}
 	return rules
 }
 
 func (p *parser) parseListOfDeclarations() (list []css_ast.Rule) {
+	list = []css_ast.Rule{}
 	for {
 		switch p.current().Kind {
 		case css_lexer.TWhitespace, css_lexer.TSemicolon:
@@ -284,7 +292,7 @@ func (p *parser) parseListOfDeclarations() (list []css_ast.Rule) {
 
 		case css_lexer.TEndOfFile, css_lexer.TCloseBrace:
 			list = p.processDeclarations(list)
-			if p.options.MangleSyntax {
+			if p.options.MinifySyntax {
 				list = mangleRules(list)
 			}
 			return
@@ -292,11 +300,12 @@ func (p *parser) parseListOfDeclarations() (list []css_ast.Rule) {
 		case css_lexer.TAtKeyword:
 			list = append(list, p.parseAtRule(atRuleContext{
 				isDeclarationList: true,
+				allowNesting:      true,
 			}))
 
 		case css_lexer.TDelimAmpersand:
 			// Reference: https://drafts.csswg.org/css-nesting-1/
-			list = append(list, p.parseSelectorRule())
+			list = append(list, p.parseSelectorRuleFrom(p.index, parseSelectorOpts{allowNesting: true}))
 
 		default:
 			list = append(list, p.parseDeclaration())
@@ -318,6 +327,28 @@ func mangleRules(rules []css_ast.Rule) []css_ast.Rule {
 			// dispatch JavaScript animation events, so removing them changes
 			// behavior: https://bugzilla.mozilla.org/show_bug.cgi?id=1004377.
 
+		case *css_ast.RAtLayer:
+			if len(r.Rules) == 0 && len(r.Names) > 0 {
+				// Do not remove empty "@layer foo {}" rules. The specification says:
+				// "Cascade layers are sorted by the order in which they first are
+				// declared, with nested layers grouped within their parent layers
+				// before any unlayered rules." So removing empty rules could change
+				// the order in which they are first declared, and is therefore invalid.
+				//
+				// We can turn "@layer foo {}" into "@layer foo;" to be shorter. But
+				// don't collapse anonymous "@layer {}" into "@layer;" because that is
+				// a syntax error.
+				r.Rules = nil
+			} else if len(r.Rules) == 1 && len(r.Names) == 1 {
+				// Only collapse layers if each layer has exactly one name
+				if r2, ok := r.Rules[0].Data.(*css_ast.RAtLayer); ok && len(r2.Names) == 1 {
+					// "@layer a { @layer b {} }" => "@layer a.b;"
+					// "@layer a { @layer b { c {} } }" => "@layer a.b { c {} }"
+					r.Names[0] = append(r.Names[0], r2.Names[0]...)
+					r.Rules = r2.Rules
+				}
+			}
+
 		case *css_ast.RKnownAt:
 			if len(r.Rules) == 0 {
 				continue
@@ -341,11 +372,20 @@ skipRule:
 	for i := n - 1; i >= 0; i-- {
 		rule := rules[i]
 
+		// Skip over preserved comments
+		next := i - 1
+		for next >= 0 {
+			if _, ok := rules[next].Data.(*css_ast.RComment); !ok {
+				break
+			}
+			next--
+		}
+
 		// Merge adjacent selectors with the same content
 		// "a { color: red; } b { color: red; }" => "a, b { color: red; }"
-		if i > 0 {
+		if next >= 0 {
 			if r, ok := rule.Data.(*css_ast.RSelector); ok {
-				if prev, ok := rules[i-1].Data.(*css_ast.RSelector); ok && css_ast.RulesEqual(r.Rules, prev.Rules) &&
+				if prev, ok := rules[next].Data.(*css_ast.RSelector); ok && css_ast.RulesEqual(r.Rules, prev.Rules) &&
 					isSafeSelectors(r.Selectors) && isSafeSelectors(prev.Selectors) {
 				nextSelector:
 					for _, sel := range r.Selectors {
@@ -487,7 +527,7 @@ var nonDeprecatedElementsSupportedByIE7 = map[string]bool{
 func isSafeSelectors(complexSelectors []css_ast.ComplexSelector) bool {
 	for _, complex := range complexSelectors {
 		for _, compound := range complex.Selectors {
-			if compound.HasNestPrefix {
+			if compound.NestingSelector != css_ast.NestingSelectorNone {
 				// Bail because this is an extension: https://drafts.csswg.org/css-nesting-1/
 				return false
 			}
@@ -579,6 +619,7 @@ const (
 	atRuleUnknown atRuleKind = iota
 	atRuleDeclarations
 	atRuleInheritContext
+	atRuleQualifiedOrEmpty
 	atRuleEmpty
 )
 
@@ -625,13 +666,37 @@ var specialAtRules = map[string]atRuleKind{
 	"document":      atRuleInheritContext,
 	"-moz-document": atRuleInheritContext,
 
+	// This is a new feature that changes how the CSS rule cascade works. It can
+	// end in either a "{}" block or a ";" rule terminator so we need this special
+	// case to support both.
+	//
+	//   Documentation: https://developer.mozilla.org/en-US/docs/Web/CSS/@layer
+	//   Motivation: https://developer.chrome.com/blog/cascade-layers/
+	//
+	"layer": atRuleQualifiedOrEmpty,
+
 	"media":    atRuleInheritContext,
 	"scope":    atRuleInheritContext,
 	"supports": atRuleInheritContext,
+
+	// Reference: https://drafts.csswg.org/css-nesting-1/
+	"nest": atRuleDeclarations,
 }
 
+type atRuleValidity uint8
+
+const (
+	atRuleInvalid atRuleValidity = iota
+	atRuleValid
+	atRuleInvalidAfter
+)
+
 type atRuleContext struct {
+	afterLoc          logger.Loc
+	charsetValidity   atRuleValidity
+	importValidity    atRuleValidity
 	isDeclarationList bool
+	allowNesting      bool
 }
 
 func (p *parser) parseAtRule(context atRuleContext) css_ast.Rule {
@@ -643,60 +708,84 @@ func (p *parser) parseAtRule(context atRuleContext) css_ast.Rule {
 
 	// Parse the prelude
 	preludeStart := p.index
+abortRuleParser:
 	switch atToken {
 	case "charset":
-		kind = atRuleEmpty
-		p.expect(css_lexer.TWhitespace)
-		if p.peek(css_lexer.TString) {
-			encoding := p.decoded()
-			if !strings.EqualFold(encoding, "UTF-8") {
-				p.log.Add(logger.Warning, &p.tracker, p.current().Range,
-					fmt.Sprintf("\"UTF-8\" will be used instead of unsupported charset %q", encoding))
+		switch context.charsetValidity {
+		case atRuleInvalid:
+			p.log.Add(logger.Warning, &p.tracker, atRange, "\"@charset\" must be the first rule in the file")
+
+		case atRuleInvalidAfter:
+			p.log.AddWithNotes(logger.Warning, &p.tracker, atRange, "\"@charset\" must be the first rule in the file",
+				[]logger.MsgData{p.tracker.MsgData(logger.Range{Loc: context.afterLoc},
+					"This rule cannot come before a \"@charset\" rule")})
+
+		case atRuleValid:
+			kind = atRuleEmpty
+			p.expect(css_lexer.TWhitespace)
+			if p.peek(css_lexer.TString) {
+				encoding := p.decoded()
+				if !strings.EqualFold(encoding, "UTF-8") {
+					p.log.Add(logger.Warning, &p.tracker, p.current().Range,
+						fmt.Sprintf("\"UTF-8\" will be used instead of unsupported charset %q", encoding))
+				}
+				p.advance()
+				p.expect(css_lexer.TSemicolon)
+				return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RAtCharset{Encoding: encoding}}
 			}
-			p.advance()
-			p.expect(css_lexer.TSemicolon)
-			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RAtCharset{Encoding: encoding}}
+			p.expect(css_lexer.TString)
 		}
-		p.expect(css_lexer.TString)
 
 	case "import":
-		kind = atRuleEmpty
-		p.eat(css_lexer.TWhitespace)
-		if path, r, ok := p.expectURLOrString(); ok {
-			importConditionsStart := p.index
-			for {
-				if kind := p.current().Kind; kind == css_lexer.TSemicolon || kind == css_lexer.TOpenBrace || kind == css_lexer.TEndOfFile {
-					break
-				}
-				p.parseComponentValue()
-			}
-			if p.current().Kind == css_lexer.TOpenBrace {
-				break // Avoid parsing an invalid "@import" rule
-			}
-			importConditions := p.convertTokens(p.tokens[importConditionsStart:p.index])
-			kind := ast.ImportAt
+		switch context.importValidity {
+		case atRuleInvalid:
+			p.log.Add(logger.Warning, &p.tracker, atRange, "\"@import\" is only valid at the top level")
 
-			// Insert or remove whitespace before the first token
-			if len(importConditions) > 0 {
-				kind = ast.ImportAtConditional
-				if p.options.RemoveWhitespace {
-					importConditions[0].Whitespace &= ^css_ast.WhitespaceBefore
-				} else {
-					importConditions[0].Whitespace |= css_ast.WhitespaceBefore
-				}
-			}
+		case atRuleInvalidAfter:
+			p.log.AddWithNotes(logger.Warning, &p.tracker, atRange, "All \"@import\" rules must come first",
+				[]logger.MsgData{p.tracker.MsgData(logger.Range{Loc: context.afterLoc},
+					"This rule cannot come before an \"@import\" rule")})
 
-			p.expect(css_lexer.TSemicolon)
-			importRecordIndex := uint32(len(p.importRecords))
-			p.importRecords = append(p.importRecords, ast.ImportRecord{
-				Kind:  kind,
-				Path:  logger.Path{Text: path},
-				Range: r,
-			})
-			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RAtImport{
-				ImportRecordIndex: importRecordIndex,
-				ImportConditions:  importConditions,
-			}}
+		case atRuleValid:
+			kind = atRuleEmpty
+			p.eat(css_lexer.TWhitespace)
+			if path, r, ok := p.expectURLOrString(); ok {
+				importConditionsStart := p.index
+				for {
+					if kind := p.current().Kind; kind == css_lexer.TSemicolon || kind == css_lexer.TOpenBrace ||
+						kind == css_lexer.TCloseBrace || kind == css_lexer.TEndOfFile {
+						break
+					}
+					p.parseComponentValue()
+				}
+				if p.current().Kind == css_lexer.TOpenBrace {
+					break // Avoid parsing an invalid "@import" rule
+				}
+				importConditions := p.convertTokens(p.tokens[importConditionsStart:p.index])
+				kind := ast.ImportAt
+
+				// Insert or remove whitespace before the first token
+				if len(importConditions) > 0 {
+					kind = ast.ImportAtConditional
+					if p.options.MinifyWhitespace {
+						importConditions[0].Whitespace &= ^css_ast.WhitespaceBefore
+					} else {
+						importConditions[0].Whitespace |= css_ast.WhitespaceBefore
+					}
+				}
+
+				p.expect(css_lexer.TSemicolon)
+				importRecordIndex := uint32(len(p.importRecords))
+				p.importRecords = append(p.importRecords, ast.ImportRecord{
+					Kind:  kind,
+					Path:  logger.Path{Text: path},
+					Range: r,
+				})
+				return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RAtImport{
+					ImportRecordIndex: importRecordIndex,
+					ImportConditions:  importConditions,
+				}}
+			}
 		}
 
 	case "keyframes", "-webkit-keyframes", "-moz-keyframes", "-ms-keyframes", "-o-keyframes":
@@ -714,22 +803,32 @@ func (p *parser) parseAtRule(context atRuleContext) css_ast.Rule {
 		}
 
 		p.eat(css_lexer.TWhitespace)
+		blockStart := p.index
+
 		if p.expect(css_lexer.TOpenBrace) {
 			var blocks []css_ast.KeyframeBlock
 
-		blocks:
+		badSyntax:
 			for {
 				switch p.current().Kind {
 				case css_lexer.TWhitespace:
 					p.advance()
 					continue
 
-				case css_lexer.TCloseBrace, css_lexer.TEndOfFile:
-					break blocks
+				case css_lexer.TCloseBrace:
+					p.advance()
+					return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RAtKeyframes{
+						AtToken: atToken,
+						Name:    name,
+						Blocks:  blocks,
+					}}
+
+				case css_lexer.TEndOfFile:
+					break badSyntax
 
 				case css_lexer.TOpenBrace:
 					p.expect(css_lexer.TPercentage)
-					p.parseComponentValue()
+					break badSyntax
 
 				default:
 					var selectors []string
@@ -742,57 +841,133 @@ func (p *parser) parseAtRule(context atRuleContext) css_ast.Rule {
 							p.advance()
 							continue
 
-						case css_lexer.TOpenBrace, css_lexer.TEndOfFile:
+						case css_lexer.TOpenBrace:
+							p.advance()
 							break selectors
 
+						case css_lexer.TCloseBrace, css_lexer.TEndOfFile:
+							p.expect(css_lexer.TOpenBrace)
+							break badSyntax
+
 						case css_lexer.TIdent, css_lexer.TPercentage:
 							text := p.decoded()
 							if t.Kind == css_lexer.TIdent {
 								if text == "from" {
-									if p.options.MangleSyntax {
+									if p.options.MinifySyntax {
 										text = "0%" // "0%" is equivalent to but shorter than "from"
 									}
 								} else if text != "to" {
 									p.expect(css_lexer.TPercentage)
 								}
-							} else if p.options.MangleSyntax && text == "100%" {
+							} else if p.options.MinifySyntax && text == "100%" {
 								text = "to" // "to" is equivalent to but shorter than "100%"
 							}
 							selectors = append(selectors, text)
 							p.advance()
 
+							// Keyframe selectors are comma-separated
+							p.eat(css_lexer.TWhitespace)
+							if p.eat(css_lexer.TComma) {
+								p.eat(css_lexer.TWhitespace)
+								if k := p.current().Kind; k != css_lexer.TIdent && k != css_lexer.TPercentage {
+									p.expect(css_lexer.TPercentage)
+									break badSyntax
+								}
+							} else if k := p.current().Kind; k != css_lexer.TOpenBrace && k != css_lexer.TCloseBrace && k != css_lexer.TEndOfFile {
+								p.expect(css_lexer.TComma)
+								break badSyntax
+							}
+
 						default:
 							p.expect(css_lexer.TPercentage)
-							p.parseComponentValue()
-						}
-
-						p.eat(css_lexer.TWhitespace)
-						if t.Kind != css_lexer.TComma && !p.peek(css_lexer.TOpenBrace) {
-							p.expect(css_lexer.TComma)
+							break badSyntax
 						}
 					}
 
-					if p.expect(css_lexer.TOpenBrace) {
-						rules := p.parseListOfDeclarations()
-						p.expect(css_lexer.TCloseBrace)
+					rules := p.parseListOfDeclarations()
+					p.expect(css_lexer.TCloseBrace)
 
-						// "@keyframes { from {} to { color: red } }" => "@keyframes { to { color: red } }"
-						if !p.options.MangleSyntax || len(rules) > 0 {
-							blocks = append(blocks, css_ast.KeyframeBlock{
-								Selectors: selectors,
-								Rules:     rules,
-							})
-						}
+					// "@keyframes { from {} to { color: red } }" => "@keyframes { to { color: red } }"
+					if !p.options.MinifySyntax || len(rules) > 0 {
+						blocks = append(blocks, css_ast.KeyframeBlock{
+							Selectors: selectors,
+							Rules:     rules,
+						})
 					}
 				}
 			}
 
+			// Otherwise, finish parsing the body and return an unknown rule
+			for !p.peek(css_lexer.TCloseBrace) && !p.peek(css_lexer.TEndOfFile) {
+				p.parseComponentValue()
+			}
 			p.expect(css_lexer.TCloseBrace)
-			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RAtKeyframes{
-				AtToken: atToken,
-				Name:    name,
-				Blocks:  blocks,
-			}}
+			prelude := p.convertTokens(p.tokens[preludeStart:blockStart])
+			block, _ := p.convertTokensHelper(p.tokens[blockStart:p.index], css_lexer.TEndOfFile, convertTokensOpts{allowImports: true})
+			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RUnknownAt{AtToken: atToken, Prelude: prelude, Block: block}}
+		}
+
+	case "nest":
+		// Reference: https://drafts.csswg.org/css-nesting-1/
+		p.eat(css_lexer.TWhitespace)
+		if kind := p.current().Kind; kind != css_lexer.TSemicolon && kind != css_lexer.TOpenBrace &&
+			kind != css_lexer.TCloseBrace && kind != css_lexer.TEndOfFile {
+			return p.parseSelectorRuleFrom(preludeStart-1, parseSelectorOpts{atNestRange: atRange, allowNesting: context.allowNesting})
+		}
+
+	case "layer":
+		// Reference: https://developer.mozilla.org/en-US/docs/Web/CSS/@layer
+
+		// Read the layer name list
+		var names [][]string
+		p.eat(css_lexer.TWhitespace)
+		if p.peek(css_lexer.TIdent) {
+			for {
+				ident, ok := p.expectValidLayerNameIdent()
+				if !ok {
+					break abortRuleParser
+				}
+				name := []string{ident}
+				for {
+					p.eat(css_lexer.TWhitespace)
+					if !p.eat(css_lexer.TDelimDot) {
+						break
+					}
+					p.eat(css_lexer.TWhitespace)
+					ident, ok := p.expectValidLayerNameIdent()
+					if !ok {
+						break abortRuleParser
+					}
+					name = append(name, ident)
+				}
+				names = append(names, name)
+				p.eat(css_lexer.TWhitespace)
+				if !p.eat(css_lexer.TComma) {
+					break
+				}
+				p.eat(css_lexer.TWhitespace)
+			}
+		}
+
+		// Read the optional block
+		if len(names) <= 1 && p.eat(css_lexer.TOpenBrace) {
+			rules := p.parseListOfRules(ruleContext{
+				parseSelectors: true,
+			})
+			p.expect(css_lexer.TCloseBrace)
+			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RAtLayer{Names: names, Rules: rules}}
+		}
+
+		// Handle lack of a block
+		if len(names) >= 1 && p.eat(css_lexer.TSemicolon) {
+			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RAtLayer{Names: names}}
+		}
+
+		// Otherwise there's some kind of syntax error
+		if kind := p.current().Kind; kind == css_lexer.TOpenBrace || kind == css_lexer.TCloseBrace || kind == css_lexer.TEndOfFile {
+			p.expect(css_lexer.TSemicolon)
+		} else {
+			p.unexpected()
 		}
 
 	default:
@@ -827,17 +1002,23 @@ prelude:
 		case css_lexer.TSemicolon, css_lexer.TCloseBrace:
 			prelude := p.convertTokens(p.tokens[preludeStart:p.index])
 
-			// Report an error for rules that should have blocks
-			if kind != atRuleEmpty && kind != atRuleUnknown {
+			switch kind {
+			case atRuleQualifiedOrEmpty:
+				// Parse a known at rule below
+				break prelude
+
+			case atRuleEmpty, atRuleUnknown:
+				// Parse an unknown at rule
+				p.expect(css_lexer.TSemicolon)
+				return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RUnknownAt{AtToken: atToken, Prelude: prelude}}
+
+			default:
+				// Report an error for rules that should have blocks
 				p.expect(css_lexer.TOpenBrace)
 				p.eat(css_lexer.TSemicolon)
 				return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RUnknownAt{AtToken: atToken, Prelude: prelude}}
 			}
 
-			// Otherwise, parse an unknown at rule
-			p.expect(css_lexer.TSemicolon)
-			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RUnknownAt{AtToken: atToken, Prelude: prelude}}
-
 		default:
 			p.parseComponentValue()
 		}
@@ -854,15 +1035,15 @@ prelude:
 		return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RUnknownAt{AtToken: atToken, Prelude: prelude, Block: block}}
 
 	case atRuleDeclarations:
-		// Parse known rules whose blocks consist of whatever the current context is
-		p.advance()
+		// Parse known rules whose blocks always consist of declarations
+		p.expect(css_lexer.TOpenBrace)
 		rules := p.parseListOfDeclarations()
 		p.expect(css_lexer.TCloseBrace)
 		return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RKnownAt{AtToken: atToken, Prelude: prelude, Rules: rules}}
 
 	case atRuleInheritContext:
 		// Parse known rules whose blocks consist of whatever the current context is
-		p.advance()
+		p.expect(css_lexer.TOpenBrace)
 		var rules []css_ast.Rule
 		if context.isDeclarationList {
 			rules = p.parseListOfDeclarations()
@@ -874,6 +1055,17 @@ prelude:
 		p.expect(css_lexer.TCloseBrace)
 		return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RKnownAt{AtToken: atToken, Prelude: prelude, Rules: rules}}
 
+	case atRuleQualifiedOrEmpty:
+		if p.eat(css_lexer.TOpenBrace) {
+			rules := p.parseListOfRules(ruleContext{
+				parseSelectors: true,
+			})
+			p.expect(css_lexer.TCloseBrace)
+			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RKnownAt{AtToken: atToken, Prelude: prelude, Rules: rules}}
+		}
+		p.expect(css_lexer.TSemicolon)
+		return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RKnownAt{AtToken: atToken, Prelude: prelude}}
+
 	default:
 		// Otherwise, parse an unknown rule
 		p.parseBlock(css_lexer.TOpenBrace, css_lexer.TCloseBrace)
@@ -882,6 +1074,21 @@ prelude:
 	}
 }
 
+func (p *parser) expectValidLayerNameIdent() (string, bool) {
+	r := p.current().Range
+	text := p.decoded()
+	if !p.expect(css_lexer.TIdent) {
+		return "", false
+	}
+	switch text {
+	case "initial", "inherit", "unset":
+		p.log.Add(logger.Warning, &p.tracker, r, fmt.Sprintf("%q cannot be used as a layer name", text))
+		p.prevError = r.Loc
+		return "", false
+	}
+	return text, true
+}
+
 func (p *parser) convertTokens(tokens []css_lexer.Token) []css_ast.Token {
 	result, _ := p.convertTokensHelper(tokens, css_lexer.TEndOfFile, convertTokensOpts{})
 	return result
@@ -897,6 +1104,38 @@ func (p *parser) convertTokensHelper(tokens []css_lexer.Token, close css_lexer.T
 	var result []css_ast.Token
 	var nextWhitespace css_ast.WhitespaceFlags
 
+	// Enable verbatim whitespace mode when the first two non-whitespace tokens
+	// are a CSS variable name followed by a colon. This is because it could be
+	// a form of CSS variable usage, and removing whitespace could potentially
+	// break this usage. For example, the following CSS is ignored by Chrome if
+	// the whitespace isn't preserved:
+	//
+	//   @supports (--foo: ) {
+	//     html { background: green; }
+	//   }
+	//
+	// Strangely whitespace removal doesn't cause the declaration to be ignored
+	// in Firefox or Safari, so there's definitely a browser bug somewhere.
+	if !opts.verbatimWhitespace {
+		for i, t := range tokens {
+			if t.Kind == css_lexer.TWhitespace {
+				continue
+			}
+			if t.Kind == css_lexer.TIdent && strings.HasPrefix(t.DecodedText(p.source.Contents), "--") {
+				for _, t := range tokens[i+1:] {
+					if t.Kind == css_lexer.TWhitespace {
+						continue
+					}
+					if t.Kind == css_lexer.TColon {
+						opts.verbatimWhitespace = true
+					}
+					break
+				}
+			}
+			break
+		}
+	}
+
 loop:
 	for len(tokens) > 0 {
 		t := tokens[0]
@@ -942,14 +1181,14 @@ loop:
 			}
 
 		case css_lexer.TNumber:
-			if p.options.MangleSyntax {
+			if p.options.MinifySyntax {
 				if text, ok := mangleNumber(token.Text); ok {
 					token.Text = text
 				}
 			}
 
 		case css_lexer.TPercentage:
-			if p.options.MangleSyntax {
+			if p.options.MinifySyntax {
 				if text, ok := mangleNumber(token.PercentageValue()); ok {
 					token.Text = text + "%"
 				}
@@ -958,7 +1197,7 @@ loop:
 		case css_lexer.TDimension:
 			token.UnitOffset = t.UnitOffset
 
-			if p.options.MangleSyntax {
+			if p.options.MinifySyntax {
 				if text, ok := mangleNumber(token.DimensionValue()); ok {
 					token.Text = text + token.DimensionUnit()
 					token.UnitOffset = uint16(len(text))
@@ -999,7 +1238,7 @@ loop:
 			token.Children = &nested
 
 			// Apply "calc" simplification rules when minifying
-			if p.options.MangleSyntax && token.Text == "calc" {
+			if p.options.MinifySyntax && token.Text == "calc" {
 				token = p.tryToReduceCalcExpression(token)
 			}
 
@@ -1031,7 +1270,7 @@ loop:
 			nested, tokens = p.convertTokensHelper(tokens, css_lexer.TCloseBrace, opts)
 
 			// Pretty-printing: insert leading and trailing whitespace when not minifying
-			if !opts.verbatimWhitespace && !p.options.RemoveWhitespace && len(nested) > 0 {
+			if !opts.verbatimWhitespace && !p.options.MinifyWhitespace && len(nested) > 0 {
 				nested[0].Whitespace |= css_ast.WhitespaceBefore
 				nested[len(nested)-1].Whitespace |= css_ast.WhitespaceAfter
 			}
@@ -1068,7 +1307,7 @@ loop:
 				}
 
 				// Assume whitespace can always be added after a comma
-				if p.options.RemoveWhitespace {
+				if p.options.MinifyWhitespace {
 					token.Whitespace &= ^css_ast.WhitespaceAfter
 					if i+1 < len(result) {
 						result[i+1].Whitespace &= ^css_ast.WhitespaceBefore
@@ -1197,15 +1436,31 @@ func mangleNumber(t string) (string, bool) {
 	return t, t != original
 }
 
-func (p *parser) parseSelectorRule() css_ast.Rule {
-	preludeStart := p.index
-
+func (p *parser) parseSelectorRuleFrom(preludeStart int, opts parseSelectorOpts) css_ast.Rule {
 	// Try parsing the prelude as a selector list
-	if list, ok := p.parseSelectorList(); ok {
-		selector := css_ast.RSelector{Selectors: list}
+	if list, ok := p.parseSelectorList(opts); ok {
+		selector := css_ast.RSelector{
+			Selectors: list,
+			HasAtNest: opts.atNestRange.Len != 0,
+		}
 		if p.expect(css_lexer.TOpenBrace) {
 			selector.Rules = p.parseListOfDeclarations()
 			p.expect(css_lexer.TCloseBrace)
+
+			// Minify "@nest" when possible
+			if p.options.MinifySyntax && selector.HasAtNest {
+				allHaveNestPrefix := true
+				for _, complex := range selector.Selectors {
+					if len(complex.Selectors) == 0 || complex.Selectors[0].NestingSelector != css_ast.NestingSelectorPrefix {
+						allHaveNestPrefix = false
+						break
+					}
+				}
+				if allHaveNestPrefix {
+					selector.HasAtNest = false
+				}
+			}
+
 			return css_ast.Rule{Loc: p.tokens[preludeStart].Range.Loc, Data: &selector}
 		}
 	}
@@ -1223,15 +1478,6 @@ loop:
 		case css_lexer.TOpenBrace, css_lexer.TEndOfFile:
 			break loop
 
-		case css_lexer.TSemicolon:
-			// Error recovery if the block is omitted (likely some CSS meta-syntax)
-			if !isAlreadyInvalid {
-				p.expect(css_lexer.TOpenBrace)
-			}
-			prelude := p.convertTokens(p.tokens[preludeStart:p.index])
-			p.advance()
-			return css_ast.Rule{Loc: preludeLoc, Data: &css_ast.RQualified{Prelude: prelude}}
-
 		default:
 			p.parseComponentValue()
 		}
@@ -1271,15 +1517,6 @@ stop:
 		case css_lexer.TEndOfFile, css_lexer.TSemicolon, css_lexer.TCloseBrace:
 			break stop
 
-		case css_lexer.TOpenBrace:
-			// Error recovery if there is an unexpected block (likely some CSS meta-syntax)
-			p.parseComponentValue()
-			p.eat(css_lexer.TWhitespace)
-			if ok && !p.peek(css_lexer.TSemicolon) {
-				p.expect(css_lexer.TSemicolon)
-			}
-			break stop
-
 		default:
 			p.parseComponentValue()
 		}
@@ -1323,15 +1560,27 @@ stop:
 
 	// Insert or remove whitespace before the first token
 	if !verbatimWhitespace && len(result) > 0 {
-		if p.options.RemoveWhitespace {
+		if p.options.MinifyWhitespace {
 			result[0].Whitespace &= ^css_ast.WhitespaceBefore
 		} else {
 			result[0].Whitespace |= css_ast.WhitespaceBefore
 		}
 	}
 
+	key := css_ast.KnownDeclarations[keyText]
+
+	// Attempt to point out trivial typos
+	if key == css_ast.DUnknown {
+		if corrected, ok := css_ast.MaybeCorrectDeclarationTypo(keyText); ok {
+			data := p.tracker.MsgData(keyToken.Range, fmt.Sprintf("%q is not a known CSS property", keyText))
+			data.Location.Suggestion = corrected
+			p.log.AddMsg(logger.Msg{Kind: logger.Warning, Data: data,
+				Notes: []logger.MsgData{{Text: fmt.Sprintf("Did you mean %q instead?", corrected)}}})
+		}
+	}
+
 	return css_ast.Rule{Loc: keyLoc, Data: &css_ast.RDeclaration{
-		Key:       css_ast.KnownDeclarations[keyText],
+		Key:       key,
 		KeyText:   keyText,
 		KeyRange:  keyToken.Range,
 		Value:     result,
