@@ -37,6 +37,16 @@ func expectPrintedTS(t *testing.T, contents string, expected string) {
 	})
 }
 
+func expectPrintedMangleTS(t *testing.T, contents string, expected string) {
+	t.Helper()
+	expectPrintedCommon(t, contents, expected, config.Options{
+		TS: config.TSOptions{
+			Parse: true,
+		},
+		MinifySyntax: true,
+	})
+}
+
 func expectPrintedTargetTS(t *testing.T, esVersion int, contents string, expected string) {
 	t.Helper()
 	expectPrintedCommon(t, contents, expected, config.Options{
@@ -136,6 +146,8 @@ func TestTSTypes(t *testing.T) {
 	expectPrintedTS(t, "let x = 'x' as keyof typeof Foo", "let x = \"x\";\n")
 	expectPrintedTS(t, "let fs: typeof import('fs') = require('fs')", "let fs = require(\"fs\");\n")
 	expectPrintedTS(t, "let fs: typeof import('fs').exists = require('fs').exists", "let fs = require(\"fs\").exists;\n")
+	expectPrintedTS(t, "let fs: typeof import('fs', { assert: { type: 'json' } }) = require('fs')", "let fs = require(\"fs\");\n")
+	expectPrintedTS(t, "let fs: typeof import('fs', { assert: { 'resolution-mode': 'import' } }) = require('fs')", "let fs = require(\"fs\");\n")
 	expectPrintedTS(t, "let x: <T>() => Foo<T>", "let x;\n")
 	expectPrintedTS(t, "let x: new <T>() => Foo<T>", "let x;\n")
 	expectPrintedTS(t, "let x: <T extends object>() => Foo<T>", "let x;\n")
@@ -395,6 +407,12 @@ func TestTSClass(t *testing.T) {
 	expectPrintedTS(t, "class Foo { foo!(): void; foo(): void {} }", "class Foo {\n  foo() {\n  }\n}\n")
 	expectParseErrorTS(t, "class Foo { foo!(): void foo(): void {} }", "<stdin>: ERROR: Expected \";\" but found \"foo\"\n")
 
+	expectPrintedTS(t, "class Foo { 'foo' = 0 }", "class Foo {\n  constructor() {\n    this[\"foo\"] = 0;\n  }\n}\n")
+	expectPrintedTS(t, "class Foo { ['foo'] = 0 }", "class Foo {\n  constructor() {\n    this[\"foo\"] = 0;\n  }\n}\n")
+	expectPrintedTS(t, "class Foo { [foo] = 0 }", "var _a;\nclass Foo {\n  constructor() {\n    this[_a] = 0;\n  }\n}\n_a = foo;\n")
+	expectPrintedMangleTS(t, "class Foo { 'foo' = 0 }", "class Foo {\n  constructor() {\n    this.foo = 0;\n  }\n}\n")
+	expectPrintedMangleTS(t, "class Foo { ['foo'] = 0 }", "class Foo {\n  constructor() {\n    this.foo = 0;\n  }\n}\n")
+
 	expectPrintedTS(t, "class Foo { foo \n ?: number }", "class Foo {\n}\n")
 	expectParseErrorTS(t, "class Foo { foo \n !: number }", "<stdin>: ERROR: Expected identifier but found \"!\"\n")
 
@@ -1117,7 +1135,7 @@ var Foo = /* @__PURE__ */ ((Foo) => {
   return Foo;
 })(Foo || {});
 var Bar = /* @__PURE__ */ ((Bar) => {
-  Bar[Bar["a"] = 10.01] = "a";
+  Bar[Bar["a"] = 10.01 /* a */] = "a";
   return Bar;
 })(Bar || {});
 `)
@@ -1147,7 +1165,7 @@ y = [0 /* A */, Foo?.["A"], Foo?.["A"]()];
 `)
 	expectPrintedTS(t, "enum Foo { Foo = 1, Bar = Foo }", `var Foo = /* @__PURE__ */ ((_Foo) => {
   _Foo[_Foo["Foo"] = 1] = "Foo";
-  _Foo[_Foo["Bar"] = 1] = "Bar";
+  _Foo[_Foo["Bar"] = 1 /* Foo */] = "Bar";
   return _Foo;
 })(Foo || {});
 `)
@@ -1159,13 +1177,13 @@ y = [0 /* A */, Foo?.["A"], Foo?.["A"]()];
 	// Check "await" and "yield"
 	expectPrintedTS(t, "enum x { await = 1, y = await }", `var x = /* @__PURE__ */ ((x) => {
   x[x["await"] = 1] = "await";
-  x[x["y"] = 1] = "y";
+  x[x["y"] = 1 /* await */] = "y";
   return x;
 })(x || {});
 `)
 	expectPrintedTS(t, "enum x { yield = 1, y = yield }", `var x = /* @__PURE__ */ ((x) => {
   x[x["yield"] = 1] = "yield";
-  x[x["y"] = 1] = "y";
+  x[x["y"] = 1 /* yield */] = "y";
   return x;
 })(x || {});
 `)
@@ -1311,8 +1329,8 @@ func TestTSDecl(t *testing.T) {
 	expectPrintedTS(t, "var a!: string, b!: boolean", "var a, b;\n")
 	expectPrintedTS(t, "let a!: string, b!: boolean", "let a, b;\n")
 	expectPrintedTS(t, "const a!: string = '', b!: boolean = false", "const a = \"\", b = false;\n")
-	expectPrintedTS(t, "var a\n!0", "var a;\ntrue;\n")
-	expectPrintedTS(t, "let a\n!0", "let a;\ntrue;\n")
+	expectPrintedTS(t, "var a\n!b", "var a;\n!b;\n")
+	expectPrintedTS(t, "let a\n!b", "let a;\n!b;\n")
 	expectParseErrorTS(t, "var a!", "<stdin>: ERROR: Expected \":\" but found end of file\n")
 	expectParseErrorTS(t, "var a! = ", "<stdin>: ERROR: Expected \":\" but found \"=\"\n")
 	expectParseErrorTS(t, "var a!, b", "<stdin>: ERROR: Expected \":\" but found \",\"\n")
@@ -1377,6 +1395,10 @@ func TestTSDeclare(t *testing.T) {
 	expectParseErrorTS(t, "function fn(x: any, ...y, ) {}", "<stdin>: ERROR: Expected \")\" but found \",\"\n")
 	expectParseErrorTS(t, "function fn(x: any, ...y: any, )", "<stdin>: ERROR: Expected \")\" but found \",\"\n")
 	expectParseErrorTS(t, "function fn(x: any, ...y: any, ) {}", "<stdin>: ERROR: Expected \")\" but found \",\"\n")
+
+	// This declares a global module
+	expectPrintedTS(t, "export as namespace ns", "")
+	expectParseErrorTS(t, "export as namespace ns.foo", "<stdin>: ERROR: Expected \";\" but found \".\"\n")
 }
 
 func TestTSDecorator(t *testing.T) {
@@ -1390,27 +1412,30 @@ func TestTSDecorator(t *testing.T) {
 	expectPrintedTS(t, "declare class Foo { foo(@dec(() => 0) x) } {let foo}", "{\n  let foo;\n}\n")
 
 	// Decorators must only work on class statements
-	expectParseErrorTS(t, "@dec enum foo {}", "<stdin>: ERROR: Expected \"class\" but found \"enum\"\n")
-	expectParseErrorTS(t, "@dec namespace foo {}", "<stdin>: ERROR: Expected \"class\" but found \"namespace\"\n")
-	expectParseErrorTS(t, "@dec function foo() {}", "<stdin>: ERROR: Expected \"class\" but found \"function\"\n")
+	notes := "<stdin>: NOTE: The preceding TypeScript decorator is here:\n" +
+		"NOTE: Decorators can only be used with class declarations in TypeScript.\n"
+	expectParseErrorTS(t, "@dec enum foo {}", "<stdin>: ERROR: Expected \"class\" after TypeScript decorator but found \"enum\"\n"+notes)
+	expectParseErrorTS(t, "@dec namespace foo {}", "<stdin>: ERROR: Expected \"class\" after TypeScript decorator but found \"namespace\"\n"+notes)
+	expectParseErrorTS(t, "@dec function foo() {}", "<stdin>: ERROR: Expected \"class\" after TypeScript decorator but found \"function\"\n"+notes)
 	expectParseErrorTS(t, "@dec abstract", "<stdin>: ERROR: Expected \"class\" but found end of file\n")
-	expectParseErrorTS(t, "@dec declare: x", "<stdin>: ERROR: Expected \"class\" but found \":\"\n")
-	expectParseErrorTS(t, "@dec declare enum foo {}", "<stdin>: ERROR: Expected \"class\" but found \"enum\"\n")
-	expectParseErrorTS(t, "@dec declare namespace foo {}", "<stdin>: ERROR: Expected \"class\" but found \"namespace\"\n")
-	expectParseErrorTS(t, "@dec declare function foo()", "<stdin>: ERROR: Expected \"class\" but found \"function\"\n")
-	expectParseErrorTS(t, "@dec export {}", "<stdin>: ERROR: Expected \"class\" but found \"{\"\n")
-	expectParseErrorTS(t, "@dec export enum foo {}", "<stdin>: ERROR: Expected \"class\" but found \"enum\"\n")
-	expectParseErrorTS(t, "@dec export namespace foo {}", "<stdin>: ERROR: Expected \"class\" but found \"namespace\"\n")
-	expectParseErrorTS(t, "@dec export function foo() {}", "<stdin>: ERROR: Expected \"class\" but found \"function\"\n")
+	expectParseErrorTS(t, "@dec declare: x", "<stdin>: ERROR: Expected \"class\" after TypeScript decorator but found \":\"\n"+notes)
+	expectParseErrorTS(t, "@dec declare enum foo {}", "<stdin>: ERROR: Expected \"class\" after TypeScript decorator but found \"enum\"\n"+notes)
+	expectParseErrorTS(t, "@dec declare namespace foo {}", "<stdin>: ERROR: Expected \"class\" after TypeScript decorator but found \"namespace\"\n"+notes)
+	expectParseErrorTS(t, "@dec declare function foo()", "<stdin>: ERROR: Expected \"class\" after TypeScript decorator but found \"function\"\n"+notes)
+	expectParseErrorTS(t, "@dec export {}", "<stdin>: ERROR: Expected \"class\" after TypeScript decorator but found \"{\"\n"+notes)
+	expectParseErrorTS(t, "@dec export enum foo {}", "<stdin>: ERROR: Expected \"class\" after TypeScript decorator but found \"enum\"\n"+notes)
+	expectParseErrorTS(t, "@dec export namespace foo {}", "<stdin>: ERROR: Expected \"class\" after TypeScript decorator but found \"namespace\"\n"+notes)
+	expectParseErrorTS(t, "@dec export function foo() {}", "<stdin>: ERROR: Expected \"class\" after TypeScript decorator but found \"function\"\n"+notes)
 	expectParseErrorTS(t, "@dec export default abstract", "<stdin>: ERROR: Expected \"class\" but found end of file\n")
-	expectParseErrorTS(t, "@dec export declare enum foo {}", "<stdin>: ERROR: Expected \"class\" but found \"enum\"\n")
-	expectParseErrorTS(t, "@dec export declare namespace foo {}", "<stdin>: ERROR: Expected \"class\" but found \"namespace\"\n")
-	expectParseErrorTS(t, "@dec export declare function foo()", "<stdin>: ERROR: Expected \"class\" but found \"function\"\n")
+	expectParseErrorTS(t, "@dec export declare enum foo {}", "<stdin>: ERROR: Expected \"class\" after TypeScript decorator but found \"enum\"\n"+notes)
+	expectParseErrorTS(t, "@dec export declare namespace foo {}", "<stdin>: ERROR: Expected \"class\" after TypeScript decorator but found \"namespace\"\n"+notes)
+	expectParseErrorTS(t, "@dec export declare function foo()", "<stdin>: ERROR: Expected \"class\" after TypeScript decorator but found \"function\"\n"+notes)
 
 	// Decorators must be forbidden outside class statements
-	expectParseErrorTS(t, "(class { @dec foo })", "<stdin>: ERROR: Expected identifier but found \"@\"\n")
-	expectParseErrorTS(t, "(class { @dec foo() {} })", "<stdin>: ERROR: Expected identifier but found \"@\"\n")
-	expectParseErrorTS(t, "(class { foo(@dec x) {} })", "<stdin>: ERROR: Expected identifier but found \"@\"\n")
+	note := "<stdin>: NOTE: This is a class expression, not a class declaration:\n"
+	expectParseErrorTS(t, "(class { @dec foo })", "<stdin>: ERROR: Decorators can only be used with class declarations in TypeScript\n"+note)
+	expectParseErrorTS(t, "(class { @dec foo() {} })", "<stdin>: ERROR: Decorators can only be used with class declarations in TypeScript\n"+note)
+	expectParseErrorTS(t, "(class { foo(@dec x) {} })", "<stdin>: ERROR: Decorators can only be used with class declarations in TypeScript\n"+note)
 	expectParseErrorTS(t, "({ @dec foo })", "<stdin>: ERROR: Expected identifier but found \"@\"\n")
 	expectParseErrorTS(t, "({ @dec foo() {} })", "<stdin>: ERROR: Expected identifier but found \"@\"\n")
 	expectParseErrorTS(t, "({ foo(@dec x) {} })", "<stdin>: ERROR: Expected identifier but found \"@\"\n")
@@ -1796,6 +1821,13 @@ func TestTSOptionalChain(t *testing.T) {
 }
 
 func TestTSJSX(t *testing.T) {
+	expectParseErrorTSX(t, "<div>></div>",
+		"<stdin>: ERROR: The character \">\" is not valid inside a JSX element\n"+
+			"NOTE: Did you mean to escape it as \"{'>'}\" instead?\n")
+	expectParseErrorTSX(t, "<div>{1}}</div>",
+		"<stdin>: ERROR: The character \"}\" is not valid inside a JSX element\n"+
+			"NOTE: Did you mean to escape it as \"{'}'}\" instead?\n")
+
 	expectPrintedTS(t, "const x = <number>1", "const x = 1;\n")
 	expectPrintedTSX(t, "const x = <number>1</number>", "const x = /* @__PURE__ */ React.createElement(\"number\", null, \"1\");\n")
 	expectParseErrorTSX(t, "const x = <number>1", "<stdin>: ERROR: Unexpected end of file\n")
@@ -1846,15 +1878,16 @@ func TestTSJSX(t *testing.T) {
 	expectPrintedTS(t, "const x = <[]>(y, z)", "const x = (y, z);\n")
 	expectPrintedTS(t, "const x = <[]>(y, z) => {}", "const x = (y, z) => {\n};\n")
 
-	expectPrintedTSX(t, "(<T>(y) => {}</T>)", "/* @__PURE__ */ React.createElement(T, null, \"(y) => \");\n")
-	expectPrintedTSX(t, "(<T extends>(y) => {}</T>)", "/* @__PURE__ */ React.createElement(T, {\n  extends: true\n}, \"(y) => \");\n")
-	expectPrintedTSX(t, "(<T extends={false}>(y) => {}</T>)", "/* @__PURE__ */ React.createElement(T, {\n  extends: false\n}, \"(y) => \");\n")
+	invalid := "<stdin>: ERROR: The character \">\" is not valid inside a JSX element\nNOTE: Did you mean to escape it as \"{'>'}\" instead?\n"
+	expectParseErrorTSX(t, "(<T>(y) => {}</T>)", invalid)
+	expectParseErrorTSX(t, "(<T extends>(y) => {}</T>)", invalid)
+	expectParseErrorTSX(t, "(<T extends={false}>(y) => {}</T>)", invalid)
 	expectPrintedTSX(t, "(<T extends X>(y) => {})", "(y) => {\n};\n")
 	expectPrintedTSX(t, "(<T extends X = Y>(y) => {})", "(y) => {\n};\n")
 	expectPrintedTSX(t, "(<T,>() => {})", "() => {\n};\n")
 	expectPrintedTSX(t, "(<T, X>(y) => {})", "(y) => {\n};\n")
 	expectPrintedTSX(t, "(<T, X>(y): (() => {}) => {})", "(y) => {\n};\n")
-	expectParseErrorTSX(t, "(<T>() => {})", "<stdin>: ERROR: Unexpected end of file\n")
+	expectParseErrorTSX(t, "(<T>() => {})", invalid+"<stdin>: ERROR: Unexpected end of file\n")
 	expectParseErrorTSX(t, "(<[]>(y))", "<stdin>: ERROR: Expected identifier but found \"[\"\n")
 	expectParseErrorTSX(t, "(<T[]>(y))", "<stdin>: ERROR: Expected \">\" but found \"[\"\n")
 	expectParseErrorTSX(t, "(<T = X>(y))", "<stdin>: ERROR: Expected \">\" but found \"=\"\n")
@@ -1911,3 +1944,26 @@ h();
 Foo[_b] = 1;
 `)
 }
+
+func TestMangleTSStringEnumLength(t *testing.T) {
+	expectPrintedTS(t, "enum x { y = '' } z = x.y.length",
+		"var x = /* @__PURE__ */ ((x) => {\n  x[\"y\"] = \"\";\n  return x;\n})(x || {});\nz = \"\" /* y */.length;\n")
+
+	expectPrintedMangleTS(t, "enum x { y = '' } x.y.length++",
+		"var x = /* @__PURE__ */ ((x) => (x.y = \"\", x))(x || {});\n\"\" /* y */.length++;\n")
+
+	expectPrintedMangleTS(t, "enum x { y = '' } x.y.length = z",
+		"var x = /* @__PURE__ */ ((x) => (x.y = \"\", x))(x || {});\n\"\" /* y */.length = z;\n")
+
+	expectPrintedMangleTS(t, "enum x { y = '' } z = x.y.length",
+		"var x = /* @__PURE__ */ ((x) => (x.y = \"\", x))(x || {});\nz = 0;\n")
+
+	expectPrintedMangleTS(t, "enum x { y = 'abc' } z = x.y.length",
+		"var x = /* @__PURE__ */ ((x) => (x.y = \"abc\", x))(x || {});\nz = 3;\n")
+
+	expectPrintedMangleTS(t, "enum x { y = 'ȧḃċ' } z = x.y.length",
+		"var x = /* @__PURE__ */ ((x) => (x.y = \"ȧḃċ\", x))(x || {});\nz = 3;\n")
+
+	expectPrintedMangleTS(t, "enum x { y = '👯‍♂️' } z = x.y.length",
+		"var x = /* @__PURE__ */ ((x) => (x.y = \"👯‍♂️\", x))(x || {});\nz = 5;\n")
+}
