@@ -39,8 +39,8 @@ func expectPrintedCommon(t *testing.T, name string, contents string, expected st
 		r := renamer.NewNoOpRenamer(symbols)
 		js := Print(tree, symbols, r, Options{
 			ASCIIOnly:           options.ASCIIOnly,
-			MangleSyntax:        options.MangleSyntax,
-			RemoveWhitespace:    options.RemoveWhitespace,
+			MinifySyntax:        options.MinifySyntax,
+			MinifyWhitespace:    options.MinifyWhitespace,
 			UnsupportedFeatures: options.UnsupportedJSFeatures,
 		}).JS
 		test.AssertEqualWithDiff(t, string(js), expected)
@@ -55,22 +55,22 @@ func expectPrinted(t *testing.T, contents string, expected string) {
 func expectPrintedMinify(t *testing.T, contents string, expected string) {
 	t.Helper()
 	expectPrintedCommon(t, contents+" [minified]", contents, expected, config.Options{
-		RemoveWhitespace: true,
+		MinifyWhitespace: true,
 	})
 }
 
 func expectPrintedMangle(t *testing.T, contents string, expected string) {
 	t.Helper()
 	expectPrintedCommon(t, contents+" [mangled]", contents, expected, config.Options{
-		MangleSyntax: true,
+		MinifySyntax: true,
 	})
 }
 
 func expectPrintedMangleMinify(t *testing.T, contents string, expected string) {
 	t.Helper()
 	expectPrintedCommon(t, contents+" [mangled, minified]", contents, expected, config.Options{
-		MangleSyntax:     true,
-		RemoveWhitespace: true,
+		MinifySyntax:     true,
+		MinifyWhitespace: true,
 	})
 }
 
@@ -84,7 +84,7 @@ func expectPrintedASCII(t *testing.T, contents string, expected string) {
 func expectPrintedMinifyASCII(t *testing.T, contents string, expected string) {
 	t.Helper()
 	expectPrintedCommon(t, contents+" [ascii]", contents, expected, config.Options{
-		RemoveWhitespace: true,
+		MinifyWhitespace: true,
 		ASCIIOnly:        true,
 	})
 }
@@ -104,7 +104,7 @@ func expectPrintedTargetMinify(t *testing.T, esVersion int, contents string, exp
 		UnsupportedJSFeatures: compat.UnsupportedJSFeatures(map[compat.Engine][]int{
 			compat.ES: {esVersion},
 		}),
-		RemoveWhitespace: true,
+		MinifyWhitespace: true,
 	})
 }
 
@@ -114,7 +114,7 @@ func expectPrintedTargetMangle(t *testing.T, esVersion int, contents string, exp
 		UnsupportedJSFeatures: compat.UnsupportedJSFeatures(map[compat.Engine][]int{
 			compat.ES: {esVersion},
 		}),
-		MangleSyntax: true,
+		MinifySyntax: true,
 	})
 }
 
@@ -156,112 +156,112 @@ func expectPrintedJSXMinify(t *testing.T, contents string, expected string) {
 			Parse:    true,
 			Preserve: true,
 		},
-		RemoveWhitespace: true,
+		MinifyWhitespace: true,
 	})
 }
 
 func TestNumber(t *testing.T) {
 	// Check "1eN"
-	expectPrinted(t, "1e-100", "1e-100;\n")
-	expectPrinted(t, "1e-4", "1e-4;\n")
-	expectPrinted(t, "1e-3", "1e-3;\n")
-	expectPrinted(t, "1e-2", "0.01;\n")
-	expectPrinted(t, "1e-1", "0.1;\n")
-	expectPrinted(t, "1e0", "1;\n")
-	expectPrinted(t, "1e1", "10;\n")
-	expectPrinted(t, "1e2", "100;\n")
-	expectPrinted(t, "1e3", "1e3;\n")
-	expectPrinted(t, "1e4", "1e4;\n")
-	expectPrinted(t, "1e100", "1e100;\n")
-	expectPrintedMinify(t, "1e-100", "1e-100;")
-	expectPrintedMinify(t, "1e-5", "1e-5;")
-	expectPrintedMinify(t, "1e-4", "1e-4;")
-	expectPrintedMinify(t, "1e-3", ".001;")
-	expectPrintedMinify(t, "1e-2", ".01;")
-	expectPrintedMinify(t, "1e-1", ".1;")
-	expectPrintedMinify(t, "1e0", "1;")
-	expectPrintedMinify(t, "1e1", "10;")
-	expectPrintedMinify(t, "1e2", "100;")
-	expectPrintedMinify(t, "1e3", "1e3;")
-	expectPrintedMinify(t, "1e4", "1e4;")
-	expectPrintedMinify(t, "1e100", "1e100;")
+	expectPrinted(t, "x = 1e-100", "x = 1e-100;\n")
+	expectPrinted(t, "x = 1e-4", "x = 1e-4;\n")
+	expectPrinted(t, "x = 1e-3", "x = 1e-3;\n")
+	expectPrinted(t, "x = 1e-2", "x = 0.01;\n")
+	expectPrinted(t, "x = 1e-1", "x = 0.1;\n")
+	expectPrinted(t, "x = 1e0", "x = 1;\n")
+	expectPrinted(t, "x = 1e1", "x = 10;\n")
+	expectPrinted(t, "x = 1e2", "x = 100;\n")
+	expectPrinted(t, "x = 1e3", "x = 1e3;\n")
+	expectPrinted(t, "x = 1e4", "x = 1e4;\n")
+	expectPrinted(t, "x = 1e100", "x = 1e100;\n")
+	expectPrintedMinify(t, "x = 1e-100", "x=1e-100;")
+	expectPrintedMinify(t, "x = 1e-5", "x=1e-5;")
+	expectPrintedMinify(t, "x = 1e-4", "x=1e-4;")
+	expectPrintedMinify(t, "x = 1e-3", "x=.001;")
+	expectPrintedMinify(t, "x = 1e-2", "x=.01;")
+	expectPrintedMinify(t, "x = 1e-1", "x=.1;")
+	expectPrintedMinify(t, "x = 1e0", "x=1;")
+	expectPrintedMinify(t, "x = 1e1", "x=10;")
+	expectPrintedMinify(t, "x = 1e2", "x=100;")
+	expectPrintedMinify(t, "x = 1e3", "x=1e3;")
+	expectPrintedMinify(t, "x = 1e4", "x=1e4;")
+	expectPrintedMinify(t, "x = 1e100", "x=1e100;")
 
 	// Check "12eN"
-	expectPrinted(t, "12e-100", "12e-100;\n")
-	expectPrinted(t, "12e-5", "12e-5;\n")
-	expectPrinted(t, "12e-4", "12e-4;\n")
-	expectPrinted(t, "12e-3", "0.012;\n")
-	expectPrinted(t, "12e-2", "0.12;\n")
-	expectPrinted(t, "12e-1", "1.2;\n")
-	expectPrinted(t, "12e0", "12;\n")
-	expectPrinted(t, "12e1", "120;\n")
-	expectPrinted(t, "12e2", "1200;\n")
-	expectPrinted(t, "12e3", "12e3;\n")
-	expectPrinted(t, "12e4", "12e4;\n")
-	expectPrinted(t, "12e100", "12e100;\n")
-	expectPrintedMinify(t, "12e-100", "12e-100;")
-	expectPrintedMinify(t, "12e-6", "12e-6;")
-	expectPrintedMinify(t, "12e-5", "12e-5;")
-	expectPrintedMinify(t, "12e-4", ".0012;")
-	expectPrintedMinify(t, "12e-3", ".012;")
-	expectPrintedMinify(t, "12e-2", ".12;")
-	expectPrintedMinify(t, "12e-1", "1.2;")
-	expectPrintedMinify(t, "12e0", "12;")
-	expectPrintedMinify(t, "12e1", "120;")
-	expectPrintedMinify(t, "12e2", "1200;")
-	expectPrintedMinify(t, "12e3", "12e3;")
-	expectPrintedMinify(t, "12e4", "12e4;")
-	expectPrintedMinify(t, "12e100", "12e100;")
+	expectPrinted(t, "x = 12e-100", "x = 12e-100;\n")
+	expectPrinted(t, "x = 12e-5", "x = 12e-5;\n")
+	expectPrinted(t, "x = 12e-4", "x = 12e-4;\n")
+	expectPrinted(t, "x = 12e-3", "x = 0.012;\n")
+	expectPrinted(t, "x = 12e-2", "x = 0.12;\n")
+	expectPrinted(t, "x = 12e-1", "x = 1.2;\n")
+	expectPrinted(t, "x = 12e0", "x = 12;\n")
+	expectPrinted(t, "x = 12e1", "x = 120;\n")
+	expectPrinted(t, "x = 12e2", "x = 1200;\n")
+	expectPrinted(t, "x = 12e3", "x = 12e3;\n")
+	expectPrinted(t, "x = 12e4", "x = 12e4;\n")
+	expectPrinted(t, "x = 12e100", "x = 12e100;\n")
+	expectPrintedMinify(t, "x = 12e-100", "x=12e-100;")
+	expectPrintedMinify(t, "x = 12e-6", "x=12e-6;")
+	expectPrintedMinify(t, "x = 12e-5", "x=12e-5;")
+	expectPrintedMinify(t, "x = 12e-4", "x=.0012;")
+	expectPrintedMinify(t, "x = 12e-3", "x=.012;")
+	expectPrintedMinify(t, "x = 12e-2", "x=.12;")
+	expectPrintedMinify(t, "x = 12e-1", "x=1.2;")
+	expectPrintedMinify(t, "x = 12e0", "x=12;")
+	expectPrintedMinify(t, "x = 12e1", "x=120;")
+	expectPrintedMinify(t, "x = 12e2", "x=1200;")
+	expectPrintedMinify(t, "x = 12e3", "x=12e3;")
+	expectPrintedMinify(t, "x = 12e4", "x=12e4;")
+	expectPrintedMinify(t, "x = 12e100", "x=12e100;")
 
 	// Check cases for "A.BeX" => "ABeY" simplification
-	expectPrinted(t, "123456789", "123456789;\n")
-	expectPrinted(t, "1123456789", "1123456789;\n")
-	expectPrinted(t, "10123456789", "10123456789;\n")
-	expectPrinted(t, "100123456789", "100123456789;\n")
-	expectPrinted(t, "1000123456789", "1000123456789;\n")
-	expectPrinted(t, "10000123456789", "10000123456789;\n")
-	expectPrinted(t, "100000123456789", "100000123456789;\n")
-	expectPrinted(t, "1000000123456789", "1000000123456789;\n")
-	expectPrinted(t, "10000000123456789", "10000000123456788;\n")
-	expectPrinted(t, "100000000123456789", "100000000123456780;\n")
-	expectPrinted(t, "1000000000123456789", "1000000000123456800;\n")
-	expectPrinted(t, "10000000000123456789", "10000000000123458e3;\n")
-	expectPrinted(t, "100000000000123456789", "10000000000012345e4;\n")
+	expectPrinted(t, "x = 123456789", "x = 123456789;\n")
+	expectPrinted(t, "x = 1123456789", "x = 1123456789;\n")
+	expectPrinted(t, "x = 10123456789", "x = 10123456789;\n")
+	expectPrinted(t, "x = 100123456789", "x = 100123456789;\n")
+	expectPrinted(t, "x = 1000123456789", "x = 1000123456789;\n")
+	expectPrinted(t, "x = 10000123456789", "x = 10000123456789;\n")
+	expectPrinted(t, "x = 100000123456789", "x = 100000123456789;\n")
+	expectPrinted(t, "x = 1000000123456789", "x = 1000000123456789;\n")
+	expectPrinted(t, "x = 10000000123456789", "x = 10000000123456788;\n")
+	expectPrinted(t, "x = 100000000123456789", "x = 100000000123456780;\n")
+	expectPrinted(t, "x = 1000000000123456789", "x = 1000000000123456800;\n")
+	expectPrinted(t, "x = 10000000000123456789", "x = 10000000000123458e3;\n")
+	expectPrinted(t, "x = 100000000000123456789", "x = 10000000000012345e4;\n")
 
 	// Check numbers around the ends of various integer ranges. These were
 	// crashing in the WebAssembly build due to a bug in the Go runtime.
 
 	// int32
-	expectPrinted(t, "0x7fff_ffff", "2147483647;\n")
-	expectPrinted(t, "0x8000_0000", "2147483648;\n")
-	expectPrinted(t, "0x8000_0001", "2147483649;\n")
-	expectPrinted(t, "-0x7fff_ffff", "-2147483647;\n")
-	expectPrinted(t, "-0x8000_0000", "-2147483648;\n")
-	expectPrinted(t, "-0x8000_0001", "-2147483649;\n")
+	expectPrinted(t, "x = 0x7fff_ffff", "x = 2147483647;\n")
+	expectPrinted(t, "x = 0x8000_0000", "x = 2147483648;\n")
+	expectPrinted(t, "x = 0x8000_0001", "x = 2147483649;\n")
+	expectPrinted(t, "x = -0x7fff_ffff", "x = -2147483647;\n")
+	expectPrinted(t, "x = -0x8000_0000", "x = -2147483648;\n")
+	expectPrinted(t, "x = -0x8000_0001", "x = -2147483649;\n")
 
 	// uint32
-	expectPrinted(t, "0xffff_ffff", "4294967295;\n")
-	expectPrinted(t, "0x1_0000_0000", "4294967296;\n")
-	expectPrinted(t, "0x1_0000_0001", "4294967297;\n")
-	expectPrinted(t, "-0xffff_ffff", "-4294967295;\n")
-	expectPrinted(t, "-0x1_0000_0000", "-4294967296;\n")
-	expectPrinted(t, "-0x1_0000_0001", "-4294967297;\n")
+	expectPrinted(t, "x = 0xffff_ffff", "x = 4294967295;\n")
+	expectPrinted(t, "x = 0x1_0000_0000", "x = 4294967296;\n")
+	expectPrinted(t, "x = 0x1_0000_0001", "x = 4294967297;\n")
+	expectPrinted(t, "x = -0xffff_ffff", "x = -4294967295;\n")
+	expectPrinted(t, "x = -0x1_0000_0000", "x = -4294967296;\n")
+	expectPrinted(t, "x = -0x1_0000_0001", "x = -4294967297;\n")
 
 	// int64
-	expectPrinted(t, "0x7fff_ffff_ffff_fdff", "9223372036854775e3;\n")
-	expectPrinted(t, "0x8000_0000_0000_0000", "9223372036854776e3;\n")
-	expectPrinted(t, "0x8000_0000_0000_3000", "9223372036854788e3;\n")
-	expectPrinted(t, "-0x7fff_ffff_ffff_fdff", "-9223372036854775e3;\n")
-	expectPrinted(t, "-0x8000_0000_0000_0000", "-9223372036854776e3;\n")
-	expectPrinted(t, "-0x8000_0000_0000_3000", "-9223372036854788e3;\n")
+	expectPrinted(t, "x = 0x7fff_ffff_ffff_fdff", "x = 9223372036854775e3;\n")
+	expectPrinted(t, "x = 0x8000_0000_0000_0000", "x = 9223372036854776e3;\n")
+	expectPrinted(t, "x = 0x8000_0000_0000_3000", "x = 9223372036854788e3;\n")
+	expectPrinted(t, "x = -0x7fff_ffff_ffff_fdff", "x = -9223372036854775e3;\n")
+	expectPrinted(t, "x = -0x8000_0000_0000_0000", "x = -9223372036854776e3;\n")
+	expectPrinted(t, "x = -0x8000_0000_0000_3000", "x = -9223372036854788e3;\n")
 
 	// uint64
-	expectPrinted(t, "0xffff_ffff_ffff_fbff", "1844674407370955e4;\n")
-	expectPrinted(t, "0x1_0000_0000_0000_0000", "18446744073709552e3;\n")
-	expectPrinted(t, "0x1_0000_0000_0000_1000", "18446744073709556e3;\n")
-	expectPrinted(t, "-0xffff_ffff_ffff_fbff", "-1844674407370955e4;\n")
-	expectPrinted(t, "-0x1_0000_0000_0000_0000", "-18446744073709552e3;\n")
-	expectPrinted(t, "-0x1_0000_0000_0000_1000", "-18446744073709556e3;\n")
+	expectPrinted(t, "x = 0xffff_ffff_ffff_fbff", "x = 1844674407370955e4;\n")
+	expectPrinted(t, "x = 0x1_0000_0000_0000_0000", "x = 18446744073709552e3;\n")
+	expectPrinted(t, "x = 0x1_0000_0000_0000_1000", "x = 18446744073709556e3;\n")
+	expectPrinted(t, "x = -0xffff_ffff_ffff_fbff", "x = -1844674407370955e4;\n")
+	expectPrinted(t, "x = -0x1_0000_0000_0000_0000", "x = -18446744073709552e3;\n")
+	expectPrinted(t, "x = -0x1_0000_0000_0000_1000", "x = -18446744073709556e3;\n")
 }
 
 func TestArray(t *testing.T) {
@@ -982,8 +982,6 @@ func TestInfinity(t *testing.T) {
 	expectPrintedMangle(t, "x = (-Infinity).toString", "x = (-1 / 0).toString;\n")
 	expectPrintedMangle(t, "x = Infinity ** 2", "x = (1 / 0) ** 2;\n")
 	expectPrintedMangle(t, "x = (-Infinity) ** 2", "x = (-1 / 0) ** 2;\n")
-	expectPrintedMangle(t, "x = ~Infinity", "x = ~(1 / 0);\n")
-	expectPrintedMangle(t, "x = ~-Infinity", "x = ~(-1 / 0);\n")
 	expectPrintedMangle(t, "x = Infinity * y", "x = 1 / 0 * y;\n")
 	expectPrintedMangle(t, "x = Infinity / y", "x = 1 / 0 / y;\n")
 	expectPrintedMangle(t, "x = y * Infinity", "x = y * (1 / 0);\n")
@@ -996,8 +994,6 @@ func TestInfinity(t *testing.T) {
 	expectPrintedMangleMinify(t, "x = (-Infinity).toString", "x=(-1/0).toString;")
 	expectPrintedMangleMinify(t, "x = Infinity ** 2", "x=(1/0)**2;")
 	expectPrintedMangleMinify(t, "x = (-Infinity) ** 2", "x=(-1/0)**2;")
-	expectPrintedMangleMinify(t, "x = ~Infinity", "x=~(1/0);")
-	expectPrintedMangleMinify(t, "x = ~-Infinity", "x=~(-1/0);")
 	expectPrintedMangleMinify(t, "x = Infinity * y", "x=1/0*y;")
 	expectPrintedMangleMinify(t, "x = Infinity / y", "x=1/0/y;")
 	expectPrintedMangleMinify(t, "x = y * Infinity", "x=y*(1/0);")
