@@ -35,9 +35,9 @@ type TSConfigJSON struct {
 	// "baseUrl" value in the "tsconfig.json" file.
 	Paths map[string][]string
 
+	TSTarget                       *config.TSTarget
 	JSXFactory                     []string
 	JSXFragmentFactory             []string
-	TSTarget                       *config.TSTarget
 	UseDefineForClassFields        config.MaybeBool
 	PreserveImportsNotUsedAsValues bool
 	PreserveValueImports           bool
@@ -139,6 +139,8 @@ func ParseTSConfigJSON(
 					constraints[compat.ES] = []int{2020}
 				case "es2021":
 					constraints[compat.ES] = []int{2021}
+				case "es2022":
+					constraints[compat.ES] = []int{2022}
 				case "esnext":
 					// Nothing to do in this case
 				default:
