@@ -68,7 +68,12 @@ let esbuildCommandAndArgs = (): [string, string[]] => {
     return ['node', [path.join(__dirname, '..', 'bin', 'esbuild')]];
   }
 
-  return [generateBinPath(), []];
+  const { binPath, isWASM } = generateBinPath()
+  if (isWASM) {
+    return ['node', [binPath]]
+  } else {
+    return [binPath, []];
+  }
 };
 
 // Return true if stderr is a TTY
@@ -250,7 +255,10 @@ let ensureServiceIsRunning = (): Service => {
 
   let { readFromStdout, afterClose, service } = common.createChannel({
     writeToStdin(bytes) {
-      child.stdin.write(bytes);
+      child.stdin.write(bytes, err => {
+        // Assume the service was stopped if we get an error writing to stdin
+        if (err) afterClose();
+      });
     },
     readFileSync: fs.readFileSync,
     isSync: false,
@@ -258,6 +266,9 @@ let ensureServiceIsRunning = (): Service => {
     esbuild: ourselves,
   });
 
+  // Assume the service was stopped if we get an error writing to stdin
+  child.stdin.on('error', afterClose);
+
   const stdin: typeof child.stdin & { unref?(): void } = child.stdin;
   const stdout: typeof child.stdout & { unref?(): void } = child.stdout;
 
@@ -564,3 +575,17 @@ let startSyncServiceWorker = () => {
 if (isInternalWorkerThread) {
   startSyncServiceWorker();
 }
+
+// Export this module's exports as an export named "default" to try to work
+// around problems due to the "default" import mess.
+//
+// More detail: When this module is converted to CommonJS, we add Babel's
+// "__esModule" marker since this module used to be ESM. However, without this
+// default export below, tools that respect the "__esModule" marker will have
+// a default export of undefined since there was no default export. This is
+// problematic because node's implementation of importing CommonJS into ESM
+// broke compatibility with the ecosystem and decided to set the "default"
+// export to "module.exports" regardless of the "__esModule" marker. I'm hoping
+// that by setting "module.exports.default = module.exports", we can hopefully
+// make this work ok in both environments.
+export default ourselves
