@@ -71,6 +71,7 @@ const buildNeutralLib = (esbuildPath) => {
   const optionalDependencies = Object.fromEntries(Object.values({
     ...platforms.exports.knownWindowsPackages,
     ...platforms.exports.knownUnixlikePackages,
+    ...platforms.exports.knownWebAssemblyFallbackPackages,
   }).sort().map(x => [x, version]))
 
   // Update "npm/esbuild/package.json"
@@ -81,6 +82,10 @@ const buildNeutralLib = (esbuildPath) => {
 }
 
 exports.buildWasmLib = async (esbuildPath) => {
+  const npmWasmShimDirs = [
+    path.join(repoDir, 'npm', 'esbuild-android-64'),
+  ]
+
   // Asynchronously start building the WebAssembly module
   const npmWasmDir = path.join(repoDir, 'npm', 'esbuild-wasm')
   const goBuildPromise = new Promise((resolve, reject) => childProcess.execFile('go',
@@ -99,12 +104,13 @@ exports.buildWasmLib = async (esbuildPath) => {
   fs.mkdirSync(esmDir, { recursive: true })
 
   // Generate "npm/esbuild-wasm/wasm_exec.js"
-  const toReplace = 'global.fs = fs;';
   const GOROOT = childProcess.execFileSync('go', ['env', 'GOROOT']).toString().trim();
   let wasm_exec_js = fs.readFileSync(path.join(GOROOT, 'misc', 'wasm', 'wasm_exec.js'), 'utf8');
-  let index = wasm_exec_js.indexOf(toReplace);
-  if (index === -1) throw new Error(`Failed to find ${JSON.stringify(toReplace)} in Go JS shim code`);
-  wasm_exec_js = wasm_exec_js.replace(toReplace, `
+  const replace = (toReplace, replacement) => {
+    if (wasm_exec_js.indexOf(toReplace) === -1) throw new Error(`Failed to find ${JSON.stringify(toReplace)} in Go JS shim code`);
+    wasm_exec_js = wasm_exec_js.replace(toReplace, replacement);
+  }
+  replace('global.fs = fs;', `
     global.fs = Object.assign({}, fs, {
       // Hack around a Unicode bug in node: https://github.com/nodejs/node/issues/24550
       write(fd, buf, offset, length, position, callback) {
@@ -130,6 +136,10 @@ exports.buildWasmLib = async (esbuildPath) => {
       },
     });
   `);
+  replace('// End of polyfills for common API.', `
+    // Make sure Go sees the shadowed "fs" global
+    const { fs } = global;
+  `);
   fs.writeFileSync(path.join(npmWasmDir, 'wasm_exec.js'), wasm_exec_js);
 
   // Generate "npm/esbuild-wasm/lib/main.js"
@@ -218,17 +228,27 @@ exports.buildWasmLib = async (esbuildPath) => {
       exit0Map[entry] = compressed.toString('base64');
     }
   }
-  fs.writeFileSync(path.join(npmWasmDir, 'exit0.js'), `
+  const exit0Code = `
 // Each of these is a native module that calls "exit(0)". This is a workaround
 // for https://github.com/nodejs/node/issues/36616. These native modules are
 // stored in a string both to make them smaller and to hide them from Yarn 2,
 // since they make Yarn 2 unzip this package.
 
 module.exports = ${JSON.stringify(exit0Map, null, 2)};
-`);
+`;
+  fs.writeFileSync(path.join(npmWasmDir, 'exit0.js'), exit0Code);
 
   // Join with the asynchronous WebAssembly build
   await goBuildPromise;
+
+  // Also copy this into the WebAssembly shim directories
+  for (const dir of npmWasmShimDirs) {
+    fs.mkdirSync(path.join(dir, 'bin'), { recursive: true })
+    fs.writeFileSync(path.join(dir, 'wasm_exec.js'), wasm_exec_js);
+    fs.writeFileSync(path.join(dir, 'exit0.js'), exit0Code);
+    fs.copyFileSync(path.join(npmWasmDir, 'bin', 'esbuild'), path.join(dir, 'bin', 'esbuild'));
+    fs.copyFileSync(path.join(npmWasmDir, 'esbuild.wasm'), path.join(dir, 'esbuild.wasm'));
+  }
 }
 
 const buildDenoLib = (esbuildPath) => {
